@@ -2,20 +2,19 @@ import numpy as np
 import os
 import unittest
 
-from os import path
-
 from test import patch_tecutil
 
 import tecplot as tp
 from tecplot.exception import *
 from tecplot.constant import *
-from tecplot.plot import Font
+from tecplot.text import Font, LabelFormat
 from tecplot.legend import ContourLegend
 
 from ..sample_data import sample_data
 
 class TestContourColormapOverride(unittest.TestCase):
     def setUp(self):
+        tp.new_layout()
         self.filename,dataset = sample_data('10x10x10')
         frame = tp.active_frame()
 
@@ -31,6 +30,7 @@ class TestContourColormapOverride(unittest.TestCase):
         self.override = frame.plot().contour(0).colormap_filter.override(0)
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
     def test_show(self):
@@ -62,6 +62,7 @@ class TestContourColormapOverride(unittest.TestCase):
 
 class TestContourColormapZebraShade(unittest.TestCase):
     def setUp(self):
+        tp.new_layout()
         self.filename,dataset = sample_data('10x10x10')
         frame = tp.active_frame()
 
@@ -77,6 +78,7 @@ class TestContourColormapZebraShade(unittest.TestCase):
         self.zebra = frame.plot().contour(0).colormap_filter.zebra_shade
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
     def test_show(self):
@@ -85,8 +87,13 @@ class TestContourColormapZebraShade(unittest.TestCase):
         self.zebra.show = False
         self.assertFalse(self.zebra.show)
 
+    def test_transparent(self):
+        for val in [True, False, True]:
+            self.zebra.transparent = val
+            self.assertEqual(self.zebra.transparent, val)
+
     def test_color(self):
-        for val in [None,Color.Black,Color.Red,None]:
+        for val in [Color.Black,Color.Red]:
             self.zebra.color = val
             self.assertEqual(self.zebra.color, val)
         with self.assertRaises(ValueError):
@@ -96,6 +103,7 @@ class TestContourColormapZebraShade(unittest.TestCase):
 
 class TestContourColormapFilter(unittest.TestCase):
     def setUp(self):
+        tp.new_layout()
         self.filename,dataset = sample_data('10x10x10')
         frame = tp.active_frame()
 
@@ -111,6 +119,7 @@ class TestContourColormapFilter(unittest.TestCase):
         self.filter = frame.plot().contour(0).colormap_filter
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
     def test_show_overrides(self):
@@ -158,8 +167,22 @@ class TestContourColormapFilter(unittest.TestCase):
             colormapfilter.fast_continuous_flood = val
             self.assertEqual(colormapfilter.fast_continuous_flood, val)
 
+    def test_continuous_min_max(self):
+        colormapfilter = self.filter
+        colormapfilter.continuous_min = -1000
+        colormapfilter.continuous_max = 1000
+        self.assertEqual(colormapfilter.continuous_min, -1000)
+        self.assertEqual(colormapfilter.continuous_max, 1000)
+        for val in [-1, 0, 0.5, 1, 100, -10]:
+            colormapfilter.continuous_min = val
+            self.assertEqual(colormapfilter.continuous_min, val)
+        for val in [-1, 0, 0.5, 1, 100]:
+            colormapfilter.continuous_max = val
+            self.assertEqual(colormapfilter.continuous_max, val)
+
 class TestColorCutoff(unittest.TestCase):
     def setUp(self):
+        tp.new_layout()
         self.filename,dataset = sample_data('10x10x10')
         frame = tp.active_frame()
 
@@ -175,17 +198,28 @@ class TestColorCutoff(unittest.TestCase):
         self.cutoff = frame.plot().contour(0).color_cutoff
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
+    def test_include_min(self):
+        for val in [True, False, True]:
+            self.cutoff.include_min = val
+            self.assertEqual(self.cutoff.include_min, val)
+
+    def test_include_max(self):
+        for val in [True, False, True]:
+            self.cutoff.include_max = val
+            self.assertEqual(self.cutoff.include_max, val)
+
     def test_min(self):
-        for val in [None,-2,-1,-0.5,0,0.5,1,2,None]:
+        for val in [-2,-1,-0.5,0,0.5,1,2]:
             self.cutoff.min = val
             self.assertEqual(self.cutoff.min, val)
         with self.assertRaises(ValueError):
             self.cutoff.min = 'badtype'
 
     def test_max(self):
-        for val in [None,-2,-1,-0.5,0,0.5,1,2,None]:
+        for val in [-2,-1,-0.5,0,0.5,1,2]:
             self.cutoff.max = val
             self.assertEqual(self.cutoff.max, val)
         with self.assertRaises(ValueError):
@@ -198,6 +232,7 @@ class TestColorCutoff(unittest.TestCase):
 
 class TestContourLabels(unittest.TestCase):
     def setUp(self):
+        tp.new_layout()
         self.filename,dataset = sample_data('10x10x10')
         frame = tp.active_frame()
 
@@ -213,6 +248,7 @@ class TestContourLabels(unittest.TestCase):
         self.labels = frame.plot().contour(0).labels
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
     def test_show(self):
@@ -261,9 +297,15 @@ class TestContourLabels(unittest.TestCase):
         with self.assertRaises(ValueError):
             contourlabels.color = 0.5
 
+    def test_filled(self):
+        contourlabels = self.labels
+        for val in [True,False,True]:
+            contourlabels.filled = val
+            self.assertEqual(contourlabels.filled, val)
+
     def test_background_color(self):
         contourlabels = self.labels
-        for val in [None,Color.Black,Color.Red,None]:
+        for val in [Color.Black,Color.Red,Color.Blue]:
             contourlabels.background_color = val
             self.assertEqual(contourlabels.background_color, val)
         with self.assertRaises(ValueError):
@@ -298,8 +340,12 @@ class TestContourLabels(unittest.TestCase):
     def test_font(self):
         self.assertIsInstance(self.labels.font, Font)
 
+    def test_format(self):
+        self.assertIsInstance(self.labels.format, LabelFormat)
+
 class TestContourLevels(unittest.TestCase):
     def setUp(self):
+        tp.new_layout()
         self.filename,dataset = sample_data('10x10x10')
         frame = tp.active_frame()
 
@@ -315,6 +361,7 @@ class TestContourLevels(unittest.TestCase):
         self.levels = frame.plot().contour(0).levels
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
     def test_list(self):
@@ -378,6 +425,7 @@ class TestContourLevels(unittest.TestCase):
 
 class TestContourLines(unittest.TestCase):
     def setUp(self):
+        tp.new_layout()
         self.filename,dataset = sample_data('10x10x10')
         frame = tp.active_frame()
 
@@ -393,6 +441,7 @@ class TestContourLines(unittest.TestCase):
         self.lines = frame.plot().contour(0).lines
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
     def test_mode(self):
@@ -439,19 +488,44 @@ class TestContourLines(unittest.TestCase):
 
 class TestContourGroup(unittest.TestCase):
     def setUp(self):
-        self.filename,dataset = sample_data('10x10x10')
+        tp.new_layout()
+        self.filename,dataset = sample_data('3x3x3_p')
         frame = tp.active_frame()
         self.dataset = frame.dataset
         self.cgroup = frame.plot().contour(0)
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
     def test_set_variable_index(self):
         self.cgroup.variable_index = 1
         self.assertEqual(self.cgroup.variable_index, 1)
-        self.cgroup.variable_index = 0
-        self.assertEqual(self.cgroup.variable_index, 0)
+        self.cgroup.levels.reset_to_nice()
+
+        expected_levels = [0.05, 0.1 , 0.15, 0.2 , 0.25, 0.3 , 0.35, 0.4 ,
+                           0.45, 0.5 , 0.55, 0.6 , 0.65, 0.7 , 0.75, 0.8 ,
+                           0.85, 0.9 , 0.95]
+
+        self.assertTrue(np.allclose(list(self.cgroup.levels), expected_levels))
+
+        self.cgroup.variable_index = 3
+        self.assertEqual(self.cgroup.variable_index, 3)
+        self.cgroup.levels.reset_to_nice()
+
+        expected_levels = [-0.8, -0.6, -0.4, -0.2, 0. , 0.2, 0.4, 0.6, 0.8, 1.,
+                           1.2, 1.4, 1.6, 1.8]
+        self.assertTrue(np.allclose(list(self.cgroup.levels), expected_levels))
+
+        with patch_tecutil('ContourSetVariableX', return_value=False):
+            with self.assertRaises(TecplotSystemError):
+                self.cgroup.variable_index = 1
+
+    def test_colormap_name1(self):
+        contourgroup = self.cgroup
+        contourgroup.colormap_name = 'Modern'
+        name = contourgroup.colormap_name
+        self.assertEqual(name, 'Modern')
 
     def test_colormap_name(self):
         contourgroup = self.cgroup
@@ -513,5 +587,5 @@ class TestContourGroup(unittest.TestCase):
         self.assertIsInstance(self.cgroup.legend, ContourLegend)
 
 if __name__ == '__main__':
-    from .. import main
-    main()
+    import test
+    test.main()
