@@ -1,26 +1,336 @@
-import numpy as np
+import collections
 import os
-import random
 import sys
 import unittest
+import warnings
 
-from os import path
-from unittest.mock import patch, Mock
-
-from test import patch_tecutil
-
+from unittest.mock import patch, Mock, PropertyMock
 import tecplot as tp
 from tecplot.exception import *
 from tecplot.constant import *
-from tecplot.data.operate import execute_equation
-from tecplot.plot import Font
-from tecplot.legend import ContourLegend
-from tecplot.tecutil import IndexRange
 
+from .. import patch_tecutil, mocked_sdk_version, skip_if_sdk_version_before
 from ..sample_data import sample_data
 
 
-class TestFieldmap(unittest.TestCase):
+class TestFieldmapCollection:
+    def test_iter(self):
+        for i, fmap in enumerate(self.fmap):
+            self.assertIsInstance(fmap, tp.plot.Fieldmap)
+            self.assertEqual(1, len(fmap.fieldmap_indices))
+            self.assertEqual(i, fmap.fieldmap_indices[0])
+        self.assertEqual(i, self.plot.num_fieldmaps - 1)
+
+    def test_getitem(self):
+        for i, fmap in enumerate(self.fmap):
+            self.assertEqual(self.fmap[i], fmap)
+
+    def test_show(self):
+        self.fmap.show = True
+        self.assertEqual(self.fmap.show, (True, True))
+        self.fmap.show = False
+        self.assertEqual(self.fmap.show, (False, False))
+        self.fmap.show = True
+        self.assertEqual(self.fmap.show, (True, True))
+
+    def test_toplevel_style_objs(self):
+        self.assertIsInstance(self.fmap.contour, tp.plot.FieldmapContour)
+        self.assertIsInstance(self.fmap.edge, tp.plot.FieldmapEdge)
+        self.assertIsInstance(self.fmap.mesh, tp.plot.FieldmapMesh)
+        self.assertIsInstance(self.fmap.scatter, tp.plot.FieldmapScatter)
+        self.assertIsInstance(self.fmap.surfaces, tp.plot.FieldmapSurfaces)
+        self.assertIsInstance(self.fmap.points, tp.plot.FieldmapPoints)
+        self.assertIsInstance(self.fmap.vector, tp.plot.FieldmapVector)
+
+    def test_eq(self):
+        fmap = self.plot.fieldmaps(0, 1)
+
+        self.assertFalse(self.fmap == self.plot.fieldmap(0))
+        self.assertTrue(self.fmap != self.plot.fieldmap(0))
+
+        self.assertFalse(self.fmap == self.plot.fieldmaps(0))
+        self.assertTrue(self.fmap != self.plot.fieldmaps(0))
+
+        self.assertTrue(self.fmap == self.plot.fieldmaps(0, 1))
+        self.assertFalse(self.fmap != self.plot.fieldmaps(0, 1))
+
+        fmaps = self.plot.fieldmaps(0)
+        fmaps += self.plot.fieldmaps(0, 1)
+        self.assertEqual(fmaps, self.plot.fieldmaps(0, 1))
+        fmaps -= self.plot.fieldmaps(1)
+        self.assertEqual(fmaps, self.plot.fieldmaps(0))
+
+    def test_group(self):
+        fmap = self.plot.fieldmaps()
+        fmap.group = 2
+        self.assertEqual(fmap.group, tuple([2, 2]))
+        fmap.group = 3
+        self.assertEqual(fmap.group, tuple([3, 3]))
+
+    def test_contour_group(self):
+        fmaps = self.plot.fieldmaps()
+        c0 = self.plot.contour(0)
+        c1 = self.plot.contour(1)
+        fmaps.contour.flood_contour_group = c0
+        indices = [g.index for g in fmaps.contour.flood_contour_group]
+        self.assertEqual(indices, [0, 0])
+        fmaps.contour.flood_contour_group = c1
+        indices = [g.index for g in fmaps.contour.flood_contour_group]
+        self.assertEqual(indices, [1, 1])
+
+    def test_line_group(self):
+        fmaps = self.plot.fieldmaps()
+        c0 = self.plot.contour(0)
+        c1 = self.plot.contour(1)
+        fmaps.contour.line_group = c0
+        indices = [g.index for g in fmaps.contour.line_group]
+        self.assertEqual(indices, [0, 0])
+        fmaps.contour.line_group = c1
+        indices = [g.index for g in fmaps.contour.line_group]
+        self.assertEqual(indices, [1, 1])
+
+    def test_mesh_color(self):
+        fmaps = self.plot.fieldmaps()
+        fmaps.mesh.color = Color.Blue
+        self.assertEqual(fmaps.mesh.color, tuple([Color.Blue] * 2))
+        fmaps.mesh.color = self.plot.contour(2)
+        color = fmaps.mesh.color
+        self.assertIsInstance(color[0], tp.plot.contour.ContourGroup)
+        self.assertIsInstance(color[1], tp.plot.contour.ContourGroup)
+        self.assertEqual(color[0].index, 2)
+        self.assertEqual(color[1].index, 2)
+
+    def test_edge_color(self):
+        fmaps = self.plot.fieldmaps()
+        fmaps.edge.color = Color.Blue
+        self.assertEqual(fmaps.edge.color, tuple([Color.Blue] * 2))
+        fmaps.edge.color = Color.Red
+        self.assertEqual(fmaps.edge.color, tuple([Color.Red] * 2))
+
+    def test_symbol(self):
+        fmaps = self.plot.fieldmaps()
+        fmaps.scatter.symbol_type = SymbolType.Geometry
+        symbol = fmaps.scatter.symbol()
+        symbol.shape = GeomShape.Sphere
+
+        self.plot.fieldmap(0).scatter.symbol_type = SymbolType.Text
+        self.plot.fieldmap(1).scatter.symbol_type = SymbolType.Geometry
+        with self.assertRaises(TecplotLogicError):
+            _ = self.plot.fieldmaps(0, 1).scatter.symbol()
+        sym = self.plot.fieldmaps(0, 1).scatter.symbol(SymbolType.Geometry)
+        sym.shape = GeomShape.Circle
+
+    def test_deprecated_show_iso_surfaces(self):
+        for val in [False, True, False]:
+            with warnings.catch_warnings(record=True) as w:
+                warnings.simplefilter('always')
+                self.fmap.show_iso_surfaces = val
+                if __debug__:
+                    assert issubclass(w[-1].category, TecplotFutureWarning)
+                else:
+                    self.assertEqual(len(w), 0)
+            with warnings.catch_warnings(record=True) as w:
+                warnings.simplefilter('always')
+                self.assertEqual(self.fmap.show_iso_surfaces, tuple([val] * 2))
+                if __debug__:
+                    assert issubclass(w[-1].category, TecplotFutureWarning)
+                else:
+                    self.assertEqual(len(w), 0)
+            fr = self.fmap.plot.frame
+            fmap3d = fr.plot(PlotType.Cartesian3D).fieldmaps(self.fmap._indices)
+            self.assertEqual(fmap3d.show_isosurfaces, tuple([val] * 2))
+
+
+class TestCartesian2DFieldmapCollection(unittest.TestCase, TestFieldmapCollection):
+    def setUp(self):
+        tp.new_layout()
+        self.filename, self.dataset = sample_data('2x2x3_overlap')
+        frame = tp.active_frame()
+
+        frame.plot_type = PlotType.Cartesian2D
+        self.plot = frame.plot()
+        contour = self.plot.contour(0)
+        contour.variable_index = self.dataset.variable('Z').index
+        self.fmap = self.plot.fieldmaps()
+        self.assertEqual(len(self.fmap.fieldmap_indices), 2)
+
+    def tearDown(self):
+        tp.new_layout()
+        os.remove(self.filename)
+
+    def test_iter(self):
+        for i, fmap in enumerate(self.fmap):
+            self.assertIsInstance(fmap, tp.plot.Fieldmap)
+            self.assertEqual(1, len(fmap.fieldmap_indices))
+            self.assertEqual(i, fmap.fieldmap_indices[0])
+        self.assertEqual(i, self.plot.num_fieldmaps - 1)
+
+    def test_contour_line_thickness(self):
+        self.fmap.contour.line_thickness = 1.0
+        self.assertAllClose(self.fmap.contour.line_thickness,
+                            [1.0] * self.plot.num_fieldmaps)
+        self.fmap.contour.line_thickness = 3.1
+        self.assertAllClose(self.fmap.contour.line_thickness,
+                            [3.1] * self.plot.num_fieldmaps)
+
+    def test_contour_line_color(self):
+        self.fmap.contour.line_color = Color.Blue
+        self.assertEqual(self.fmap.contour.line_color,
+                         tuple([Color.Blue] * self.plot.num_fieldmaps))
+
+    def test_edge_border(self):
+        self.fmap.edge.i_border = BorderLocation.Both
+        self.assertEqual(self.fmap.edge.i_border,
+                         tuple([BorderLocation.Both] * self.plot.num_fieldmaps))
+        self.fmap.edge.i_border = None
+        self.assertEqual(self.fmap.edge.i_border,
+                         tuple([BorderLocation.None_] * self.plot.num_fieldmaps))
+
+    def test_mesh_color(self):
+        self.fmap.mesh.color = Color.Blue
+        self.assertEqual(self.fmap.mesh.color,
+                         tuple([Color.Blue] * self.plot.num_fieldmaps))
+        self.fmap.mesh.color = self.plot.contour(0)
+        self.assertEqual(self.fmap.mesh.color,
+                         tuple([self.plot.contour(0)] * self.plot.num_fieldmaps))
+
+    def test_points_step_single(self):
+        pts = list(self.fmap)[0].points
+        self.assertIsInstance(pts.step, tp.session.style.IJK)
+
+        pts.step = (2,3,4)
+        self.assertEqual(pts.step, (2,3,4))
+
+        pts.step = (1,1,1)
+        self.assertEqual(pts.step, (1,1,1))
+
+        pts.step = (1,1,1)
+        pts.step.i = 5
+        self.assertEqual(pts.step.i, 5)
+        self.assertEqual(pts.step, (5,1,1))
+
+        pts.step = (1,1,1)
+        pts.step.j = 5
+        self.assertEqual(pts.step.j, 5)
+        self.assertEqual(pts.step, (1,5,1))
+
+        pts.step = (1,1,1)
+        pts.step.k = 5
+        self.assertEqual(pts.step.k, 5)
+        self.assertEqual(pts.step, (1,1,5))
+
+    def test_points_step_collection(self):
+        pts = self.fmap.points
+        self.assertIsInstance(pts.step, tp.session.style.IJK)
+        pts.step = (1, 1, 1)
+        self.assertIsInstance(pts.step[0], tuple)
+        self.assertIsInstance(pts.step[1], tuple)
+        self.assertIsInstance(pts.step[2], tuple)
+
+        pts.step = (2,3,4)
+        self.assertEqual(tuple([tuple(x) for x in pts.step]),
+                         ((2,2), (3,3), (4,4)))
+
+        pts.step = (1,1,1)
+        self.assertEqual(tuple([tuple(x) for x in pts.step]),
+                         ((1,1), (1,1), (1,1)))
+
+        pts.step = (1,1,1)
+        pts.step.i = 5
+        self.assertEqual(pts.step.i, (5,5))
+        self.assertEqual(tuple([tuple(x) for x in pts.step]),
+                         ((5,5), (1,1), (1,1)))
+
+        pts.step = (1,1,1)
+        pts.step.j = 5
+        self.assertEqual(pts.step.j, (5,5))
+        self.assertEqual(tuple([tuple(x) for x in pts.step]),
+                         ((1,1), (5,5), (1,1)))
+
+    def test_scatter_color(self):
+        self.fmap.scatter.color = Color.Blue
+        self.assertEqual(self.fmap.scatter.color,
+                         tuple([Color.Blue] * self.plot.num_fieldmaps))
+        self.fmap.scatter.color = self.plot.contour(0)
+        self.assertEqual(self.fmap.scatter.color,
+                         tuple([self.plot.contour(0)] * self.plot.num_fieldmaps))
+        self.fmap.scatter.color = Color.Blue
+        self.assertEqual(self.fmap.scatter.color,
+                         tuple([Color.Blue] * self.plot.num_fieldmaps))
+
+    def test_deprecated_show_slices(self):
+        for val in [False, True, False]:
+            with warnings.catch_warnings(record=True) as w:
+                warnings.simplefilter('always')
+                self.fmap.show_slices = val
+                if __debug__:
+                    assert issubclass(w[-1].category, TecplotFutureWarning)
+                else:
+                    self.assertEqual(len(w), 0)
+            with warnings.catch_warnings(record=True) as w:
+                warnings.simplefilter('always')
+                self.assertEqual(self.fmap.show_slices, tuple([val] * 2))
+                if __debug__:
+                    assert issubclass(w[-1].category, TecplotFutureWarning)
+                else:
+                    self.assertEqual(len(w), 0)
+            fr = self.fmap.plot.frame
+            fmap3d = fr.plot(PlotType.Cartesian3D).fieldmaps(self.fmap._indices)
+            self.assertEqual(fmap3d.show_slices, tuple([val] * 2))
+
+    def test_deprecated_show_streamtraces(self):
+        for val in [False, True, False]:
+            with warnings.catch_warnings(record=True) as w:
+                warnings.simplefilter('always')
+                self.fmap.show_streamtraces = val
+                if __debug__:
+                    assert issubclass(w[-1].category, TecplotFutureWarning)
+                else:
+                    self.assertEqual(len(w), 0)
+            with warnings.catch_warnings(record=True) as w:
+                warnings.simplefilter('always')
+                self.assertEqual(self.fmap.show_streamtraces, tuple([val] * 2))
+                if __debug__:
+                    assert issubclass(w[-1].category, TecplotFutureWarning)
+                else:
+                    self.assertEqual(len(w), 0)
+            fr = self.fmap.plot.frame
+            fmap3d = fr.plot(PlotType.Cartesian3D).fieldmaps(self.fmap._indices)
+            self.assertEqual(fmap3d.show_streamtraces, tuple([val] * 2))
+
+
+class TestCartesian3DFieldmapCollection(unittest.TestCase, TestFieldmapCollection):
+    def setUp(self):
+        tp.new_layout()
+        self.filename, self.dataset = sample_data('2x2x3_overlap')
+        frame = tp.active_frame()
+        self.plot = frame.plot()
+        contour = self.plot.contour(0)
+        contour.variable_index = self.dataset.variable('Z').index
+        self.fmap = self.plot.fieldmaps()
+        self.assertEqual(len(self.fmap.fieldmap_indices), 2)
+
+    def tearDown(self):
+        tp.new_layout()
+        os.remove(self.filename)
+
+    def test_show_isosurfaces(self):
+        for val in [False, True, False]:
+            self.fmap.show_isosurfaces = val
+            self.assertEqual(self.fmap.show_isosurfaces, tuple([val] * 2))
+
+    def test_show_slices(self):
+        for val in [False, True, False]:
+            self.fmap.show_slices = val
+            self.assertEqual(self.fmap.show_slices, tuple([val] * 2))
+
+    def test_show_streamtraces(self):
+        for val in [False, True, False]:
+            self.fmap.show_streamtraces = val
+            self.assertEqual(self.fmap.show_streamtraces, tuple([val] * 2))
+
+
+class TestFieldmapCollectionValidity(unittest.TestCase):
     def setUp(self):
         tp.new_layout()
         self.filename, self.dataset = sample_data('2x2x3_overlap')
@@ -30,11 +340,23 @@ class TestFieldmap(unittest.TestCase):
         self.plot = frame.plot()
         contour = self.plot.contour(0)
         contour.variable_index = self.dataset.variable('Z').index
-        self.fmap = self.plot.fieldmap(0)
+        self.fmap = self.plot.fieldmaps()
+        self.assertEqual(len(self.fmap.fieldmap_indices), 2)
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
+    def test_delete_while_iterating(self):
+        fmap_itr = iter(self.fmap)
+        fmap = next(fmap_itr)
+        self.assertIsInstance(fmap, tp.plot.Fieldmap)
+        self.dataset.delete_zones(self.dataset.num_zones - 1)
+        fmap = next(fmap_itr)
+        with self.assertRaises((TecplotLogicError, TecplotSystemError)):
+            _ = list(fmap.zones)
+
+class TestFieldmap:
     def test_show(self):
         for val in [True, False, True]:
             self.fmap.show = val
@@ -51,21 +373,11 @@ class TestFieldmap(unittest.TestCase):
         with self.assertRaises(TypeError):
             self.fmap.group = None
 
-    def test_show_volume_objs(self):
-        for val in [False, True, False]:
-            self.fmap.show_iso_surfaces = val
-            self.fmap.show_slices = val
-            self.fmap.show_streamtraces = val
-            self.assertEqual(self.fmap.show_iso_surfaces, val)
-            self.assertEqual(self.fmap.show_slices, val)
-            self.assertEqual(self.fmap.show_streamtraces, val)
-
     def test_toplevel_style_objs(self):
         self.assertIsInstance(self.fmap.contour, tp.plot.FieldmapContour)
         self.assertIsInstance(self.fmap.edge, tp.plot.FieldmapEdge)
         self.assertIsInstance(self.fmap.mesh, tp.plot.FieldmapMesh)
         self.assertIsInstance(self.fmap.scatter, tp.plot.FieldmapScatter)
-        self.assertIsInstance(self.fmap.shade, tp.plot.FieldmapShade3D)
         self.assertIsInstance(self.fmap.surfaces, tp.plot.FieldmapSurfaces)
         self.assertIsInstance(self.fmap.points, tp.plot.FieldmapPoints)
         self.assertIsInstance(self.fmap.vector, tp.plot.FieldmapVector)
@@ -73,6 +385,10 @@ class TestFieldmap(unittest.TestCase):
     def test_zones(self):
         self.assertEqual(list(self.fmap.zones), [self.dataset.zone(0)])
 
+        with patch_tecutil('FieldMapGetZones', return_value=(False, None)):
+            with self.assertRaises(TecplotSystemError):
+                _ = list(self.fmap.zones)
+
     def test_eq(self):
         fmap0 = self.plot.fieldmap(0)
         self.assertTrue(fmap0 == self.plot.fieldmap(0))
@@ -81,23 +397,79 @@ class TestFieldmap(unittest.TestCase):
         self.assertFalse(fmap0 != self.plot.fieldmap(0))
 
 
+class TestCartesian2DFieldmap(unittest.TestCase, TestFieldmap):
+    def setUp(self):
+        tp.new_layout()
+        self.filename, self.dataset = sample_data('2x2x3_overlap')
+        frame = tp.active_frame()
+
+        frame.plot_type = PlotType.Cartesian2D
+        self.plot = frame.plot()
+        contour = self.plot.contour(0)
+        contour.variable_index = self.dataset.variable('Z').index
+        self.fmap = self.plot.fieldmap(0)
+
+    def tearDown(self):
+        tp.new_layout()
+        os.remove(self.filename)
+
+    def test_shade(self):
+        self.assertIsInstance(self.fmap.shade, tp.plot.FieldmapShade)
+
+
+class TestCartesian3DFieldmap(unittest.TestCase, TestFieldmap):
+    def setUp(self):
+        tp.new_layout()
+        self.filename, self.dataset = sample_data('2x2x3_overlap')
+        frame = tp.active_frame()
+
+        frame.plot_type = PlotType.Cartesian3D
+        self.plot = frame.plot()
+        contour = self.plot.contour(0)
+        contour.variable_index = self.dataset.variable('Z').index
+        self.fmap = self.plot.fieldmap(0)
+
+    def tearDown(self):
+        tp.new_layout()
+        os.remove(self.filename)
+
+    def test_shade(self):
+        self.assertIsInstance(self.fmap.shade, tp.plot.FieldmapShade3D)
+
+    def test_show_isosurfaces(self):
+        for val in [False, True, False]:
+            self.fmap.show_isosurfaces = val
+            self.assertEqual(self.fmap.show_isosurfaces, val)
+
+    def test_show_slices(self):
+        for val in [False, True, False]:
+            self.fmap.show_slices = val
+            self.assertEqual(self.fmap.show_slices, val)
+
+    def test_show_streamtraces(self):
+        for val in [False, True, False]:
+            self.fmap.show_streamtraces = val
+            self.assertEqual(self.fmap.show_streamtraces, val)
+
+
 class TestFieldmapContour(unittest.TestCase):
     def setUp(self):
         tp.new_layout()
-        self.filename, dataset = sample_data('2x2x3_overlap')
+        self.filename, self.dataset = sample_data('2x2x3_overlap')
         frame = tp.active_frame()
 
         # we need to set up the frame/plot/etc etc manually here
         frame.plot_type = PlotType.Cartesian3D
         self.plot = frame.plot()
         contour = self.plot.contour(0)
-        contour.variable_index = dataset.variable('Z').index
+        contour.variable_index = self.dataset.variable('Z').index
         fmap = self.plot.fieldmap(0)
         fmap.surfaces.surfaces_to_plot = SurfacesToPlot.BoundaryFaces
         self.plot.show_contour = True
         self.contour = fmap.contour
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
     def test_eq(self):
@@ -127,23 +499,32 @@ class TestFieldmapContour(unittest.TestCase):
             ci = self.plot.contour(i)
             cj = self.plot.contour(j)
             ck = self.plot.contour(k)
-            contour.flood_group = ci
+            contour.flood_contour_group = ci
             contour.line_group = cj
             contour.line_color = ck
-            self.assertEqual(contour.flood_group, ci)
+            self.assertEqual(contour.flood_contour_group, ci)
             self.assertEqual(contour.line_group, cj)
             self.assertEqual(contour.line_color, ck)
 
+    def test_setting_rgb_contour_group(self):
+        self.plot.rgb_coloring.red_variable = self.dataset.variable(0)
+        self.plot.rgb_coloring.green_variable = self.dataset.variable(1)
+        self.plot.rgb_coloring.blue_variable = self.dataset.variable(2)
+
+        self.contour.flood_contour_group = self.plot.rgb_coloring
+        self.assertIsInstance(self.contour.flood_contour_group,
+                              type(self.plot.rgb_coloring))
+
     def test_setting_contour_groups_by_index(self):
         contour = self.contour
         for i,j in [[1,2],[2,3],[3,5]]:
             ci = self.plot.contour(i)
             cj = self.plot.contour(j)
-            contour.flood_group_index = i
+            contour.flood_contour_group_index = i
             contour.line_group_index = j
-            self.assertEqual(contour.flood_group_index, i)
+            self.assertEqual(contour.flood_contour_group_index, i)
             self.assertEqual(contour.line_group_index, j)
-            self.assertEqual(contour.flood_group, ci)
+            self.assertEqual(contour.flood_contour_group, ci)
             self.assertEqual(contour.line_group, cj)
 
     def test_line_color(self):
@@ -153,6 +534,15 @@ class TestFieldmapContour(unittest.TestCase):
         with self.assertRaises(ValueError):
             self.contour.line_color = 0.5
 
+    def test_rgb_line_color(self):
+        self.plot.rgb_coloring.red_variable = self.dataset.variable(0)
+        self.plot.rgb_coloring.green_variable = self.dataset.variable(1)
+        self.plot.rgb_coloring.blue_variable = self.dataset.variable(2)
+
+        self.contour.line_color = self.plot.rgb_coloring
+        self.assertIsInstance(self.contour.line_color,
+                              type(self.plot.rgb_coloring))
+
     def test_line_pattern(self):
         contour = self.contour
         for val in [LinePattern.Solid,LinePattern.Dashed]:
@@ -185,11 +575,12 @@ class TestFieldmapContour(unittest.TestCase):
         with self.assertRaises(TecplotSystemError):
             contour.pattern_length = -1
 
-    def test_lighting_effect(self):
+    def test_use_lighting_effect(self):
         contour = self.contour
         for val in [True,False]:
-            contour.lighting_effect = val
-            self.assertEqual(contour.lighting_effect, val)
+            contour.use_lighting_effect = val
+            self.assertEqual(contour.use_lighting_effect, val)
+
 
 class TestFieldmapEdge(unittest.TestCase):
     def setUp(self):
@@ -208,6 +599,7 @@ class TestFieldmapEdge(unittest.TestCase):
         self.edge = fmap.edge
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
     def test_color(self):
@@ -290,6 +682,7 @@ class TestFieldmapEffects(unittest.TestCase):
         self.effects = fmap.effects
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
     def test_instance(self):
@@ -302,14 +695,28 @@ class TestFieldmapEffects(unittest.TestCase):
             self.assertEqual(effects.value_blanking, val)
 
     def test_clip_planes(self):
-        effects = self.effects
-        for val in [None,[],0,1,[1,2],[3,4,5],list(range(6))]:
-            effects.clip_planes = val
-            if val == []:
-                val = None
-            elif isinstance(val, int):
-                val = [val]
-            self.assertEqual(effects.clip_planes, val)
+        with patch('tecplot.plot.FieldmapEffects3D.clip_planes',
+                   PropertyMock()) as clip_planes:
+            with warnings.catch_warnings(record=True) as w:
+                warnings.simplefilter('always')
+                self.effects.clip_planes = [0,1]
+                if sys.version_info < (3,):
+                    clip_planes.assert_called()
+                else:
+                    clip_planes.assert_called_once()
+                if __debug__:
+                    self.assertEqual(len(w), 1)
+                    assert issubclass(w[-1].category, TecplotFutureWarning)
+                clip_planes.reset_mock()
+                _ = self.effects.clip_planes
+                if sys.version_info < (3,):
+                    clip_planes.assert_called()
+                else:
+                    clip_planes.assert_called_once()
+                if __debug__:
+                    self.assertEqual(len(w), 2)
+                    assert issubclass(w[-1].category, TecplotFutureWarning)
+
 
 class TestFieldmapEffects3D(unittest.TestCase):
     def setUp(self):
@@ -324,14 +731,21 @@ class TestFieldmapEffects3D(unittest.TestCase):
         self.effects = fmap.effects
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
     def test_instance(self):
         self.assertIsInstance(self.effects, tp.plot.FieldmapEffects3D)
 
+    def test_use_translucency(self):
+        effects = self.effects
+        for val in [True, False, True]:
+            effects.use_translucency = val
+            self.assertEqual(effects.use_translucency, val)
+
     def test_surface_translucency(self):
         effects = self.effects
-        for val in [None,1,99]:
+        for val in [1,5,99]:
             effects.surface_translucency = val
             self.assertEqual(effects.surface_translucency, val)
         with self.assertRaises(ValueError):
@@ -351,6 +765,49 @@ class TestFieldmapEffects3D(unittest.TestCase):
         with self.assertRaises(ValueError):
             effects.lighting_effect = 0.5
 
+    def test_validate_clip_planes(self):
+        if __debug__:
+            slc0 = self.effects.fieldmap.plot.slice(0)
+            slc1 = self.effects.fieldmap.plot.slice(1)
+            slc1.orientation = SliceSurface.Arbitrary
+            with warnings.catch_warnings(record=True) as w:
+                warnings.simplefilter('always')
+                self.effects._validate_clip_planes([0])
+                self.assertEqual(len(w), 0)
+                self.effects._validate_clip_planes([1])
+                self.assertEqual(len(w), 1)
+                assert issubclass(w[-1].category, TecplotWarning)
+                self.effects.clip_planes = [0]
+                self.effects._validate_clip_planes()
+                self.assertEqual(len(w), 1)
+                self.effects.clip_planes = [1]
+                self.assertEqual(len(w), 2)
+                self.effects._validate_clip_planes()
+                self.assertEqual(len(w), 3)
+                assert issubclass(w[-1].category, TecplotWarning)
+
+    def test_clip_planes(self):
+        self.effects.clip_planes = None
+        self.assertIsNone(self.effects.clip_planes)
+        self.effects.clip_planes = 0
+        self.assertEqual(self.effects.clip_planes, [0])
+        self.effects.clip_planes = [1]
+        self.assertEqual(self.effects.clip_planes, [1])
+        self.effects.clip_planes = [0,1]
+        self.assertEqual(self.effects.clip_planes, [0,1])
+        self.effects.clip_planes = self.plot.slice(0)
+        self.assertEqual(self.effects.clip_planes, [0])
+        self.effects.clip_planes = [self.plot.slice(0)]
+        self.assertEqual(self.effects.clip_planes, [0])
+        self.effects.clip_planes = self.plot.slices(0, 2)
+        self.assertEqual(self.effects.clip_planes, [0, 2])
+        self.effects.clip_planes = [self.plot.slices(0, 2)]
+        self.assertEqual(self.effects.clip_planes, [0, 2])
+        if __debug__:
+            with self.assertRaises(TecplotIndexError):
+                self.effects.clip_planes = 7
+
+
 class TestFieldmapMesh(unittest.TestCase):
     def setUp(self):
         tp.new_layout()
@@ -368,6 +825,7 @@ class TestFieldmapMesh(unittest.TestCase):
         self.mesh = fmap.mesh
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
     def test_show(self):
@@ -440,6 +898,7 @@ class TestFieldmapPoints(unittest.TestCase):
         self.pts = fmap.points
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
     def test_points_to_plot(self):
@@ -452,15 +911,33 @@ class TestFieldmapPoints(unittest.TestCase):
             self.pts.points_to_plot = 0.5
 
     def test_step(self):
+        IJK = collections.namedtuple('IJK', ('i', 'j', 'k'))
+        IJK.__new__.__defaults__ = (None, None, None)
+
+
+        self.pts.step = (2,3,4)
+        self.assertEqual(self.pts.step, (2,3,4))
+        self.pts.step = (1,1,1)
+        self.assertEqual(self.pts.step, (1,1,1))
+
+        self.pts.step.i = 5
+        self.assertEqual(self.pts.step.i, 5)
+        self.pts.step.j = 5
+        self.assertEqual(self.pts.step.j, 5)
+        self.pts.step.k = 7
+        self.assertEqual(self.pts.step.k, 7)
+
+        self.pts.step = (1,1,1)
         for val in [None,1,2,(1,),(2,),(2,3),(4,5,6)]:
             self.pts.step = val
             if isinstance(val, tuple):
-                step = tp.plot.FieldmapPoints._IJKStep(*val)
+                step = IJK(*val)
             else:
-                step = tp.plot.FieldmapPoints._IJKStep(val)
-            tup = (step.i or 1, step.j or 1, step.k or 1)
+                step = IJK(val)
+            tup = tuple([step.i or 1, step.j or 1, step.k or 1])
             self.assertEqual(self.pts.step, tup)
 
+
 class TestFieldmapScatter(unittest.TestCase):
     def setUp(self):
         tp.new_layout()
@@ -478,6 +955,7 @@ class TestFieldmapScatter(unittest.TestCase):
         self.scatter = fmap.scatter
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
     def test_show(self):
@@ -498,9 +976,16 @@ class TestFieldmapScatter(unittest.TestCase):
         self.scatter.symbol_type = SymbolType.Geometry
         self.assertIsInstance(self.scatter.symbol(), tp.plot.GeometrySymbol)
 
+    def test_fill_mode(self):
+        for c in [FillMode.UseSpecificColor, FillMode.UseLineColor,
+                  FillMode.UseBackgroundColor]:
+            self.scatter.fill_mode = c
+            self.assertEqual(self.scatter.fill_mode, c)
+        with self.assertRaises(ValueError):
+            self.scatter.fill_mode = 0.5
+
     def test_fill_color(self):
-        for c in [Color.Red, Color.Black, self.plot.contour(1),
-                  FillMode.UseLineColor, FillMode.UseBackgroundColor]:
+        for c in [Color.Red, Color.Black, self.plot.contour(1)]:
             self.scatter.fill_color = c
             self.assertEqual(self.scatter.fill_color, c)
         with self.assertRaises(ValueError):
@@ -561,6 +1046,7 @@ class TestFieldmapShade(unittest.TestCase):
         self.shade = fmap.shade
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
     def test_instance(self):
@@ -601,6 +1087,7 @@ class TestFieldmapShade3D(unittest.TestCase):
         self.shade = fmap.shade
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
     def test_instance(self):
@@ -620,11 +1107,11 @@ class TestFieldmapShade3D(unittest.TestCase):
             shade.show = val
             self.assertEqual(shade.show, val)
 
-    def test_lighting_effect(self):
+    def test_use_lighting_effect(self):
         shade = self.shade
         for val in [True,False]:
-            shade.lighting_effect = val
-            self.assertEqual(shade.lighting_effect, val)
+            shade.use_lighting_effect = val
+            self.assertEqual(shade.use_lighting_effect, val)
 
 class TestFieldmapSurfaces(unittest.TestCase):
     def setUp(self):
@@ -643,6 +1130,7 @@ class TestFieldmapSurfaces(unittest.TestCase):
         self.srf = fmap.surfaces
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
     def test_surfaces_to_plot(self):
@@ -657,55 +1145,56 @@ class TestFieldmapSurfaces(unittest.TestCase):
 
     def test_ranges(self):
         self.srf.i_range = None,None,None
-        self.assertEqual(self.srf.i_range, IndexRange(0,-1,1))
+
+        self.assertEqual(self.srf.i_range, (0,-1,1))
         self.srf.i_range = 0,-1,1
-        self.assertEqual(self.srf.i_range, IndexRange(0,-1,1))
+        self.assertEqual(self.srf.i_range, (0,-1,1))
         self.srf.i_range = -10,-1,1
-        self.assertEqual(self.srf.i_range, IndexRange(-10,-1,1))
+        self.assertEqual(self.srf.i_range, (-10,-1,1))
         self.srf.i_range = None,10,1
-        self.assertEqual(self.srf.i_range, IndexRange(0,10,1))
+        self.assertEqual(self.srf.i_range, (0,10,1))
         self.srf.i_range = 0,None,1
-        self.assertEqual(self.srf.i_range, IndexRange(0,-1,1))
+        self.assertEqual(self.srf.i_range, (0,-1,1))
         self.srf.i_range = 0,10,None
-        self.assertEqual(self.srf.i_range, IndexRange(0,10,1))
+        self.assertEqual(self.srf.i_range, (0,10,1))
         self.srf.i_range = 0,10
-        self.assertEqual(self.srf.i_range, IndexRange(0,10,1))
+        self.assertEqual(self.srf.i_range, (0,10,1))
         self.srf.i_range = 0,
-        self.assertEqual(self.srf.i_range, IndexRange(0,-1,1))
+        self.assertEqual(self.srf.i_range, (0,-1,1), str(self.srf.i_range))
 
         self.srf.j_range = None,None,None
-        self.assertEqual(self.srf.j_range, IndexRange(0,-1,1))
+        self.assertEqual(self.srf.j_range, (0,-1,1))
         self.srf.j_range = 0,-1,1
-        self.assertEqual(self.srf.j_range, IndexRange(0,-1,1))
+        self.assertEqual(self.srf.j_range, (0,-1,1))
         self.srf.j_range = -10,-1,1
-        self.assertEqual(self.srf.j_range, IndexRange(-10,-1,1))
+        self.assertEqual(self.srf.j_range, (-10,-1,1))
         self.srf.j_range = None,10,1
-        self.assertEqual(self.srf.j_range, IndexRange(0,10,1))
+        self.assertEqual(self.srf.j_range, (0,10,1))
         self.srf.j_range = 0,None,1
-        self.assertEqual(self.srf.j_range, IndexRange(0,-1,1))
+        self.assertEqual(self.srf.j_range, (0,-1,1))
         self.srf.j_range = 0,10,None
-        self.assertEqual(self.srf.j_range, IndexRange(0,10,1))
+        self.assertEqual(self.srf.j_range, (0,10,1))
         self.srf.j_range = 0,10
-        self.assertEqual(self.srf.j_range, IndexRange(0,10,1))
+        self.assertEqual(self.srf.j_range, (0,10,1))
         self.srf.j_range = 0,
-        self.assertEqual(self.srf.j_range, IndexRange(0,-1,1))
+        self.assertEqual(self.srf.j_range, (0,-1,1))
 
         self.srf.k_range = None,None,None
-        self.assertEqual(self.srf.k_range, IndexRange(0,-1,1))
+        self.assertEqual(self.srf.k_range, (0,-1,1))
         self.srf.k_range = 0,-1,1
-        self.assertEqual(self.srf.k_range, IndexRange(0,-1,1))
+        self.assertEqual(self.srf.k_range, (0,-1,1))
         self.srf.k_range = -10,-1,1
-        self.assertEqual(self.srf.k_range, IndexRange(-10,-1,1))
+        self.assertEqual(self.srf.k_range, (-10,-1,1))
         self.srf.k_range = None,10,1
-        self.assertEqual(self.srf.k_range, IndexRange(0,10,1))
+        self.assertEqual(self.srf.k_range, (0,10,1))
         self.srf.k_range = 0,None,1
-        self.assertEqual(self.srf.k_range, IndexRange(0,-1,1))
+        self.assertEqual(self.srf.k_range, (0,-1,1))
         self.srf.k_range = 0,10,None
-        self.assertEqual(self.srf.k_range, IndexRange(0,10,1))
+        self.assertEqual(self.srf.k_range, (0,10,1))
         self.srf.k_range = 0,10
-        self.assertEqual(self.srf.k_range, IndexRange(0,10,1))
+        self.assertEqual(self.srf.k_range, (0,10,1))
         self.srf.k_range = 0,
-        self.assertEqual(self.srf.k_range, IndexRange(0,-1,1))
+        self.assertEqual(self.srf.k_range, (0,-1,1))
 
     def test_range_limits(self):
         maxint = 2**64 - 1            # maximum of unsigned int64
@@ -726,13 +1215,9 @@ class TestFieldmapSurfaces(unittest.TestCase):
                 self.srf.i_range = 0,minint - 1
         else:
             if sys.version_info < (3,):
-                exp = OverflowError
-                with self.assertRaises(exp):
-                    self.srf.i_range = maxint + 1,
+                exp = TecplotSystemError
                 with self.assertRaises(exp):
                     self.srf.i_range = minint - 1,
-                with self.assertRaises(exp):
-                    self.srf.i_range = 0,maxint + 1
                 with self.assertRaises(exp):
                     self.srf.i_range = 0,minint - 1
             else:
@@ -760,6 +1245,7 @@ class TestFieldmapVector(unittest.TestCase):
         self.vector = fmap.vector
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
     def test_arrowhead_style(self):
@@ -823,5 +1309,5 @@ class TestFieldmapVector(unittest.TestCase):
             self.vector.vector_type = 0.5
 
 if __name__ == '__main__':
-    from .. import main
-    main()
+    import test
+    test.main()
