@@ -1,23 +1,159 @@
-import numpy as np
 import os
-import random
-import sys
 import unittest
 
-from os import path
+from unittest.mock import patch
 
-from test import patch_tecutil
+import numpy as np
 
 import tecplot as tp
+from tecplot.tecutil import sv
 from tecplot.exception import *
 from tecplot.constant import *
-from tecplot.data.operate import execute_equation
-from tecplot.plot import Font
-from tecplot.legend import ContourLegend
-from tecplot.tecutil import IndexRange
+from tecplot import tecutil
+from tecplot.plot.axis import PolarAngleLineAxis, RadialLineAxis
+from tecplot.session import IndexRange
 
+from test import assert_style
 from ..sample_data import sample_data
 
+
+class TestLinemapCollection(unittest.TestCase):
+    def setUp(self):
+        tp.new_layout()
+        self.filename,self.dataset = sample_data('xylines_poly')
+        frame = tp.active_frame()
+        frame.plot_type = PlotType.XYLine
+        plot = frame.plot()
+        zone = self.dataset.zone(0)
+        x = self.dataset.variable('X')
+        plot.delete_linemaps()
+        plot.add_linemap('p', zone, x, self.dataset.variable('P'))
+        plot.add_linemap('q', zone, x, self.dataset.variable('Q'))
+        plot.add_linemap('r', zone, x, self.dataset.variable('R'))
+
+        self.plot = plot
+        self.lmaps = plot.linemaps()
+
+    def tearDown(self):
+        tp.new_layout()
+        os.remove(self.filename)
+
+    def test_linemap_indices(self):
+        self.assertEqual(self.lmaps.linemap_indices, [0, 1, 2])
+
+    def test_iter(self):
+        for i, lmap in enumerate(self.lmaps):
+            self.assertEqual(i, lmap.index)
+
+    def test_eq(self):
+        for i, lmap in enumerate(self.lmaps):
+            self.assertEqual(self.plot.linemap(i), lmap)
+
+    def test_add(self):
+        lmaps = self.plot.linemaps()
+        lmaps -= 1
+        self.assertEqual(lmaps.linemap_indices, [0, 2])
+        lmaps -= self.plot.linemap(0)
+        self.assertEqual(lmaps.linemap_indices, [2])
+        lmaps -= 'r'
+        self.assertEqual(lmaps.linemap_indices, [])
+
+        lmaps += 0
+        self.assertEqual(lmaps.linemap_indices, [0])
+        lmaps += self.plot.linemap('q')
+        self.assertEqual(lmaps.linemap_indices, [0, 1])
+        lmaps += 'r'
+        self.assertEqual(lmaps.linemap_indices, [0, 1, 2])
+
+    def test_show(self):
+        self.lmaps.show = True
+        self.assertEqual(self.lmaps.show, (True, True, True))
+        self.lmaps.show = False
+        self.assertEqual(self.lmaps.show, (False, False, False))
+
+    def test_show_in_legend(self):
+        for val in [LegendShow.Always,LegendShow.Never,LegendShow.Auto]:
+            self.lmaps.show_in_legend = val
+            self.assertEqual(self.lmaps.show_in_legend,
+                             tuple([val] * self.plot.num_linemaps))
+
+    def test_name(self):
+        self.lmaps.name = 'test'
+        self.assertEqual(self.lmaps.name,
+                         tuple(['test'] * self.plot.num_linemaps))
+
+    def test_toplevel_style_objs(self):
+        self.assertIsInstance(self.lmaps.bars, tp.plot.LinemapBars)
+        self.assertIsInstance(self.lmaps.curve, tp.plot.LinemapCurve)
+        self.assertIsInstance(self.lmaps.error_bars, tp.plot.LinemapErrorBars)
+        self.assertIsInstance(self.lmaps.indices, tp.plot.LinemapIndices)
+        self.assertIsInstance(self.lmaps.line, tp.plot.LinemapLine)
+        self.assertIsInstance(self.lmaps.symbols, tp.plot.LinemapSymbols)
+
+    def test_sort(self):
+        with self.assertRaises(TecplotLogicError):
+            self.lmaps.sort_mode = LineMapSort.BySpecificVar
+        self.lmaps.sort_variable = self.dataset.variable(0)
+        for val in [LineMapSort.ByIndependentVar, LineMapSort.ByDependentVar,
+                    LineMapSort.BySpecificVar]:
+            self.lmaps.sort_mode = val
+            self.assertEqual(self.lmaps.sort_mode,
+                             tuple([val] * self.plot.num_linemaps))
+        with self.assertRaises(ValueError):
+            self.lmaps.sort_mode = 0.5
+
+        var0 = self.dataset.variable(0)
+        var2 = self.dataset.variable(2)
+        var3 = self.dataset.variable(3)
+        for val in [var0, var2, var3]:
+            self.lmaps.sort_variable = val
+            self.assertEqual(self.lmaps.sort_variable,
+                             tuple([val] * self.plot.num_linemaps))
+        self.lmaps.sort_variable_index = 2
+        self.assertEqual(self.lmaps.sort_variable_index,
+                         tuple([2] * self.plot.num_linemaps))
+        self.assertEqual(self.lmaps.sort_variable,
+                         tuple([var2] * self.plot.num_linemaps))
+        with self.assertRaises(AttributeError):
+            self.lmaps.sort_variable = 0.5
+
+    def test_symbols(self):
+        sym = self.lmaps.symbols
+        self.assertIsInstance(sym.symbol(SymbolType.Text), tp.plot.TextSymbol)
+        self.assertIsInstance(sym.symbol(SymbolType.Geometry), tp.plot.GeometrySymbol)
+
+        sym.symbol_type = SymbolType.Geometry
+        sym.symbol().shape = GeomShape.Square
+
+        self.assertEqual(sym.symbol().shape,
+                         tuple([GeomShape.Square] * self.plot.num_linemaps))
+
+        self.plot.linemap(0).symbols.symbol_type = SymbolType.Text
+        self.plot.linemap(1).symbols.symbol_type = SymbolType.Geometry
+        with self.assertRaises(TecplotLogicError):
+            _ = sym.symbol()
+
+    def test_zones(self):
+        z0 = self.dataset.zone(0)
+        self.lmaps.zone = z0
+        self.assertEqual(self.lmaps.zone_index, (0, 0, 0))
+        self.assertEqual(self.lmaps.zone, (z0, z0, z0))
+        self.lmaps.zone_index = 0
+        self.assertEqual(self.lmaps.zone_index, (0, 0, 0))
+        self.assertEqual(self.lmaps.zone, (z0, z0, z0))
+
+        with patch('tecplot.session.style.get_style') as g, \
+             patch('tecplot.session.style.set_style') as s:
+            self.lmaps.zone_index = 0
+            _ = self.lmaps.zone_index
+            g.assert_called_with(
+                tecutil.Index, sv.LINEMAP, sv.ASSIGN, sv.ZONE,
+                UNIQUEID=self.lmaps.plot.frame.uid, OFFSET1=2)
+            self.assertEqual(g.call_count, 3)
+            s.assert_called_once_with(
+                tecutil.Index(0), sv.LINEMAP, sv.ASSIGN, sv.ZONE,
+                UNIQUEID=self.lmaps.plot.frame.uid, OBJECTSET={0, 1, 2})
+
 class TestLinemap(unittest.TestCase):
     def setUp(self):
         tp.new_layout()
@@ -31,6 +167,7 @@ class TestLinemap(unittest.TestCase):
         self.lmap = frame.plot().linemap(0)
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
     def test_eq(self):
@@ -45,7 +182,7 @@ class TestLinemap(unittest.TestCase):
             self.assertEqual(self.lmap.show, val)
 
     def test_show_in_legend(self):
-        for val in [LegendShow.Yes,LegendShow.No,LegendShow.Auto]:
+        for val in [LegendShow.Always,LegendShow.Never,LegendShow.Auto]:
             self.lmap.show_in_legend = val
             self.assertEqual(self.lmap.show_in_legend, val)
         with self.assertRaises(ValueError):
@@ -67,12 +204,27 @@ class TestLinemap(unittest.TestCase):
         self.assertEqual(self.lmap.index, 0)
 
     def test_sort(self):
-        for val in [LineMapSort.IndependentVar,
-                    LineMapSort.DependentVar, self.dataset.variable('R')]:
-            self.lmap.sort_by = val
-            self.assertEqual(self.lmap.sort_by, val)
+        with self.assertRaises(TecplotLogicError):
+            self.lmap.sort_mode = LineMapSort.BySpecificVar
+        self.lmap.sort_variable = self.dataset.variable(0)
+        for val in [LineMapSort.ByIndependentVar, LineMapSort.ByDependentVar,
+                    LineMapSort.BySpecificVar]:
+            self.lmap.sort_mode = val
+            self.assertEqual(self.lmap.sort_mode, val)
         with self.assertRaises(ValueError):
-            self.lmap.sort_by = 0.5
+            self.lmap.sort_mode = 0.5
+
+        var0 = self.dataset.variable(0)
+        var2 = self.dataset.variable(2)
+        var3 = self.dataset.variable(3)
+        for val in [var0, var2, var3]:
+            self.lmap.sort_variable = val
+            self.assertEqual(self.lmap.sort_variable, val)
+        self.lmap.sort_variable_index = 2
+        self.assertEqual(self.lmap.sort_variable_index, 2)
+        self.assertEqual(self.lmap.sort_variable, var2)
+        with self.assertRaises(AttributeError):
+            self.lmap.sort_variable = 0.5
 
     def test_zone(self):
         self.lmap.zone = self.dataset.zone(0)
@@ -82,6 +234,22 @@ class TestLinemap(unittest.TestCase):
         self.assertEqual(self.lmap.zone_index, 0)
         self.assertEqual(self.lmap.zone, self.dataset.zone(0))
 
+        with patch('tecplot.session.style.get_style') as g, \
+             patch('tecplot.session.style.set_style') as s:
+            self.lmap.zone_index = 0
+            _ = self.lmap.zone_index
+            g.assert_called_once_with(
+                tecutil.Index, sv.LINEMAP, sv.ASSIGN, sv.ZONE,
+                UNIQUEID=self.lmap.plot.frame.uid, OFFSET1=0)
+            s.assert_called_once_with(
+                tecutil.Index(0), sv.LINEMAP, sv.ASSIGN, sv.ZONE,
+                UNIQUEID=self.lmap.plot.frame.uid, OBJECTSET={0})
+
+    def test_aux_data(self):
+        self.lmap.aux_data['test'] = 'some info'
+        self.assertEqual(self.lmap.aux_data['test'], 'some info')
+
+
 class TestLinemapBars(unittest.TestCase):
     def setUp(self):
         tp.new_layout()
@@ -95,6 +263,7 @@ class TestLinemapBars(unittest.TestCase):
         self.bars = frame.plot().linemap(0).bars
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
     def test_eq(self):
@@ -110,9 +279,16 @@ class TestLinemapBars(unittest.TestCase):
         with self.assertRaises(ValueError):
             self.bars.line_color = 0.5
 
+    def test_fill_mode(self):
+        for c in [FillMode.UseSpecificColor, FillMode.UseBackgroundColor,
+                  FillMode.UseLineColor]:
+            self.bars.fill_mode = c
+            self.assertEqual(self.bars.fill_mode, c)
+        with self.assertRaises(ValueError):
+            self.bars.fill_mode = 0.5
+
     def test_fill_color(self):
-        for c in [Color.Red, Color.Black, Color.Blue,
-                  FillMode.UseBackgroundColor, FillMode.UseLineColor]:
+        for c in [Color.Red, Color.Black, Color.Blue, Color.Blue]:
             self.bars.fill_color = c
             self.assertEqual(self.bars.fill_color, c)
         with self.assertRaises(ValueError):
@@ -143,6 +319,7 @@ class TestLinemapBars(unittest.TestCase):
         with self.assertRaises(TecplotSystemError):
             self.bars.size = -1
 
+
 class TestLinemapCurve(unittest.TestCase):
     def setUp(self):
         tp.new_layout()
@@ -152,30 +329,33 @@ class TestLinemapCurve(unittest.TestCase):
         plot = frame.plot()
         zone = self.dataset.zone(0)
         x = self.dataset.variable('X')
+        p = self.dataset.variable('P')
+        p.values(zone)[:] = np.abs(p.values(zone)[:])
+        plot.delete_linemaps()
         plot.add_linemap('p', zone, x, self.dataset.variable('P'))
         self.curve = frame.plot().linemap(0).curve
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
     def test_curve_type(self):
-        for val in [CurveType.LineSeg,CurveType.PolynomialFit,
-                    CurveType.EToRFit]:
+        for val in [CurveType.LineSeg, CurveType.PolynomialFit,
+                    CurveType.EToRFit, CurveType.LineSeg]:
             self.curve.curve_type = val
             self.assertEqual(self.curve.curve_type, val)
         with self.assertRaises(ValueError):
             self.curve.curve_type = 0.5
 
+    def test_use_fit_range(self):
+        for val in [True, False, True]:
+            self.curve.use_fit_range = val
+            self.assertEqual(self.curve.use_fit_range, val)
+
     def test_fit_range(self):
-        for val in [(-10,0),(0,10),(0,-10),None]:
+        for val in [(-10,0),(0,10),(0,-10)]:
             self.curve.fit_range = val
             self.assertEqual(self.curve.fit_range, val)
-            if __debug__:
-                with self.assertRaises(TecplotLogicError):
-                    self.curve.fit_range = 3
-            else:
-                with self.assertRaises(TypeError):
-                    self.curve.fit_range = 3
 
     def test_num_points(self):
         for val in [2,10,1000]:
@@ -207,26 +387,28 @@ class TestLinemapCurve(unittest.TestCase):
         with self.assertRaises(ValueError):
             self.curve.polynomial_order = 'badtype'
 
+    def test_clamp_spline(self):
+        for val in [True, False, True]:
+            self.curve.clamp_spline = val
+            self.assertEqual(self.curve.clamp_spline, val)
+
     def test_spline_derivative_at_ends(self):
-        for val in [(-10,0),(0,10),(0,-10),None]:
+        for val in [(-10,0),(0,10),(0,-10)]:
             self.curve.spline_derivative_at_ends = val
             self.assertEqual(self.curve.spline_derivative_at_ends, val)
-            if __debug__:
-                with self.assertRaises(TecplotLogicError):
-                    self.curve.spline_derivative_at_ends = 3
-            else:
-                with self.assertRaises(TypeError):
-                    self.curve.spline_derivative_at_ends = 3
 
     def test_weight_variable(self):
+        for val in [True, False, True]:
+            self.curve.use_weight_variable = val
+            self.assertEqual(self.curve.use_weight_variable, val)
         xvar = self.dataset.variable('X')
         yvar = self.dataset.variable('Y')
         self.curve.weight_variable = yvar
         self.assertEqual(self.curve.weight_variable, yvar)
-        self.curve.weight_variable = None
-        self.assertEqual(self.curve.weight_variable, None)
         self.curve.weight_variable = xvar
         self.assertEqual(self.curve.weight_variable, xvar)
+        with self.assertRaises(AttributeError):
+            self.curve.weight_variable = None
 
     def test_weight_variable_index(self):
         var0 = self.dataset.variable(0)
@@ -234,12 +416,12 @@ class TestLinemapCurve(unittest.TestCase):
         self.curve.weight_variable_index = 1
         self.assertEqual(self.curve.weight_variable_index, 1)
         self.assertEqual(self.curve.weight_variable, var1)
-        self.curve.weight_variable_index = None
-        self.assertEqual(self.curve.weight_variable_index, None)
-        self.assertEqual(self.curve.weight_variable, None)
         self.curve.weight_variable_index = 0
         self.assertEqual(self.curve.weight_variable_index, 0)
         self.assertEqual(self.curve.weight_variable, var0)
+        with self.assertRaises(TypeError):
+            self.curve.weight_variable_index = None
+
 
 class TestLinemapErrorBars(unittest.TestCase):
     def setUp(self):
@@ -255,6 +437,7 @@ class TestLinemapErrorBars(unittest.TestCase):
         self.error_bars.variable = self.dataset.variable('X')
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
     def test_bar_type(self):
@@ -335,6 +518,7 @@ class TestLinemapErrorBars(unittest.TestCase):
         self.assertEqual(self.error_bars.variable_index, 0)
         self.assertEqual(self.error_bars.variable, var0)
 
+
 class TestLinemapIndices(unittest.TestCase):
     def setUp(self):
         tp.new_layout()
@@ -348,6 +532,7 @@ class TestLinemapIndices(unittest.TestCase):
         self.indices = frame.plot().linemap(0).indices
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
     def test_varying_index(self):
@@ -359,55 +544,56 @@ class TestLinemapIndices(unittest.TestCase):
 
     def test_ranges(self):
         self.indices.i_range = None,None,None
-        self.assertEqual(self.indices.i_range, IndexRange(0,-1,1))
+        self.assertEqual(self.indices.i_range, (0,-1,1))
         self.indices.i_range = 0,-1,1
-        self.assertEqual(self.indices.i_range, IndexRange(0,-1,1))
+        self.assertEqual(self.indices.i_range, (0,-1,1))
         self.indices.i_range = -10,-1,1
-        self.assertEqual(self.indices.i_range, IndexRange(-10,-1,1))
+        self.assertEqual(self.indices.i_range, (-10,-1,1))
         self.indices.i_range = None,10,1
-        self.assertEqual(self.indices.i_range, IndexRange(0,10,1))
+        self.assertEqual(self.indices.i_range, (0,10,1))
         self.indices.i_range = 0,None,1
-        self.assertEqual(self.indices.i_range, IndexRange(0,-1,1))
+        self.assertEqual(self.indices.i_range, (0,-1,1))
         self.indices.i_range = 0,10,None
-        self.assertEqual(self.indices.i_range, IndexRange(0,10,1))
+        self.assertEqual(self.indices.i_range, (0,10,1))
         self.indices.i_range = 0,10
-        self.assertEqual(self.indices.i_range, IndexRange(0,10,1))
+        self.assertEqual(self.indices.i_range, (0,10,1))
         self.indices.i_range = 0,
-        self.assertEqual(self.indices.i_range, IndexRange(0,-1,1))
+        self.assertEqual(self.indices.i_range, (0,-1,1))
 
         self.indices.j_range = None,None,None
-        self.assertEqual(self.indices.j_range, IndexRange(0,-1,1))
+        self.assertEqual(self.indices.j_range, (0,-1,1))
         self.indices.j_range = 0,-1,1
-        self.assertEqual(self.indices.j_range, IndexRange(0,-1,1))
+        self.assertEqual(self.indices.j_range, (0,-1,1))
         self.indices.j_range = -10,-1,1
-        self.assertEqual(self.indices.j_range, IndexRange(-10,-1,1))
+        self.assertEqual(self.indices.j_range, (-10,-1,1))
         self.indices.j_range = None,10,1
-        self.assertEqual(self.indices.j_range, IndexRange(0,10,1))
+        self.assertEqual(self.indices.j_range, (0,10,1))
         self.indices.j_range = 0,None,1
-        self.assertEqual(self.indices.j_range, IndexRange(0,-1,1))
+        self.assertEqual(self.indices.j_range, (0,-1,1))
         self.indices.j_range = 0,10,None
-        self.assertEqual(self.indices.j_range, IndexRange(0,10,1))
+        self.assertEqual(self.indices.j_range, (0,10,1))
         self.indices.j_range = 0,10
-        self.assertEqual(self.indices.j_range, IndexRange(0,10,1))
+        self.assertEqual(self.indices.j_range, (0,10,1))
         self.indices.j_range = 0,
-        self.assertEqual(self.indices.j_range, IndexRange(0,-1,1))
+        self.assertEqual(self.indices.j_range, (0,-1,1))
 
         self.indices.k_range = None,None,None
-        self.assertEqual(self.indices.k_range, IndexRange(0,-1,1))
+        self.assertEqual(self.indices.k_range, (0,-1,1))
         self.indices.k_range = 0,-1,1
-        self.assertEqual(self.indices.k_range, IndexRange(0,-1,1))
+        self.assertEqual(self.indices.k_range, (0,-1,1))
         self.indices.k_range = -10,-1,1
-        self.assertEqual(self.indices.k_range, IndexRange(-10,-1,1))
+        self.assertEqual(self.indices.k_range, (-10,-1,1))
         self.indices.k_range = None,10,1
-        self.assertEqual(self.indices.k_range, IndexRange(0,10,1))
+        self.assertEqual(self.indices.k_range, (0,10,1))
         self.indices.k_range = 0,None,1
-        self.assertEqual(self.indices.k_range, IndexRange(0,-1,1))
+        self.assertEqual(self.indices.k_range, (0,-1,1))
         self.indices.k_range = 0,10,None
-        self.assertEqual(self.indices.k_range, IndexRange(0,10,1))
+        self.assertEqual(self.indices.k_range, (0,10,1))
         self.indices.k_range = 0,10
-        self.assertEqual(self.indices.k_range, IndexRange(0,10,1))
+        self.assertEqual(self.indices.k_range, (0,10,1))
         self.indices.k_range = 0,
-        self.assertEqual(self.indices.k_range, IndexRange(0,-1,1))
+        self.assertEqual(self.indices.k_range, (0,-1,1))
+
 
 class TestLinemapLine(unittest.TestCase):
     def setUp(self):
@@ -422,6 +608,7 @@ class TestLinemapLine(unittest.TestCase):
         self.line = frame.plot().linemap(0).line
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
     def test_show(self):
@@ -465,6 +652,7 @@ class TestLinemapLine(unittest.TestCase):
         with self.assertRaises(TecplotSystemError):
             self.line.pattern_length = -1
 
+
 class TestLinemapSymbols(unittest.TestCase):
     def setUp(self):
         tp.new_layout()
@@ -478,6 +666,7 @@ class TestLinemapSymbols(unittest.TestCase):
         self.sym = frame.plot().linemap(0).symbols
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
     def test_show(self):
@@ -521,9 +710,16 @@ class TestLinemapSymbols(unittest.TestCase):
         else:
             self.sym.step = -1
 
-    def test_fill_color(self):
-        for c in [Color.Red, Color.Black, FillMode.UseLineColor,
+    def test_fill_mode(self):
+        for c in [FillMode.UseSpecificColor, FillMode.UseLineColor,
                   FillMode.UseBackgroundColor]:
+            self.sym.fill_mode = c
+            self.assertEqual(self.sym.fill_mode, c)
+        with self.assertRaises(ValueError):
+            self.sym.color = 0.5
+
+    def test_fill_color(self):
+        for c in [Color.Red, Color.Black, Color.Blue]:
             self.sym.fill_color = c
             self.assertEqual(self.sym.fill_color, c)
         with self.assertRaises(ValueError):
@@ -556,6 +752,7 @@ class TestLinemapSymbols(unittest.TestCase):
         with self.assertRaises(TecplotSystemError):
             self.sym.line_thickness = -1
 
+
 class TestXYLinemap(unittest.TestCase):
     def setUp(self):
         tp.new_layout()
@@ -569,6 +766,7 @@ class TestXYLinemap(unittest.TestCase):
         self.lmap = self.plot.linemap(0)
 
     def tearDown(self):
+        tp.new_layout()
         os.remove(self.filename)
 
     def test_function_dependency(self):
@@ -580,7 +778,7 @@ class TestXYLinemap(unittest.TestCase):
         with self.assertRaises(ValueError):
             self.lmap.function_dependency = 0.5
 
-    def test_axis(self):
+    def test_axes(self):
         for i in [0,1,4]:
             self.lmap.x_axis_index = i
             self.lmap.y_axis_index = i
@@ -596,7 +794,7 @@ class TestXYLinemap(unittest.TestCase):
             self.assertEqual(self.lmap.y_axis, a)
             self.assertEqual(self.lmap.y_axis_index, a.index)
 
-    def test_variable(self):
+    def test_variables(self):
         for i in [0,1,2]:
             self.lmap.x_variable_index = i
             self.lmap.y_variable_index = i
@@ -611,6 +809,52 @@ class TestXYLinemap(unittest.TestCase):
             self.assertEqual(self.lmap.y_variable, v)
             self.assertEqual(self.lmap.y_variable_index, v.index)
 
+
+class TestPolarLinemap(unittest.TestCase):
+    def setUp(self):
+        tp.new_layout()
+        self.filename,self.dataset = sample_data('xylines_poly')
+        frame = tp.active_frame()
+        frame.plot_type = PlotType.PolarLine
+        self.plot = frame.plot()
+        zone = self.dataset.zone(0)
+        x = self.dataset.variable('X')
+        self.plot.add_linemap('p', zone, x, self.dataset.variable('P'))
+        self.lmap = self.plot.linemap(0)
+
+    def tearDown(self):
+        tp.new_layout()
+        os.remove(self.filename)
+
+    def test_function_dependency(self):
+        for var in [FunctionDependency.ThetaIndependent,
+                    FunctionDependency.RIndependent,
+                    FunctionDependency.ThetaIndependent]:
+            self.lmap.function_dependency = var
+            self.assertEqual(self.lmap.function_dependency, var)
+        with self.assertRaises(ValueError):
+            self.lmap.function_dependency = 0.5
+
+    def test_axes(self):
+        self.assertIsInstance(self.lmap.r_axis, RadialLineAxis)
+        self.assertIsInstance(self.lmap.theta_axis, PolarAngleLineAxis)
+
+    def test_variables(self):
+        for i in [0,1,2]:
+            self.lmap.r_variable_index = i
+            self.lmap.theta_variable_index = i
+            self.assertEqual(self.lmap.r_variable_index, i)
+            self.assertEqual(self.lmap.theta_variable_index, i)
+        for i in [0,1,2]:
+            v = self.dataset.variable(i)
+            self.lmap.r_variable = v
+            self.assertEqual(self.lmap.r_variable, v)
+            self.assertEqual(self.lmap.r_variable_index, v.index)
+            self.lmap.theta_variable = v
+            self.assertEqual(self.lmap.theta_variable, v)
+            self.assertEqual(self.lmap.theta_variable_index, v.index)
+
+
 if __name__ == '__main__':
-    from .. import main
-    main()
+    import test
+    test.main()
